﻿/*
 * Created by SharpDevelop.
 * User: Pawel
 * Date: 2017-11-21
 * Time: 10:29
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using System.IO.Ports;
using System.IO;
using System.Text;
using System.Threading;
using System.Text.RegularExpressions;
using System.Diagnostics;
using System.Resources;
using System.Reflection;
using B1Client;



namespace B1_Standalone_Configurator
{
	/// <summary>
	/// Description of MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		private B1Module B1Module;
		static SerialPort ComPort;
		byte [] TableUID = new byte[11];
		bool SearchStarted = false;
		
		byte lastCommand;
		
		private const byte GetModuleVersion = 1;
		private const byte CheckVersion = 2;
		private const byte GetUID = 3;
		private const byte ReadUidFromMemory = 4;

		
		public MainForm()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			COMPortsAvailabe();
			B1Module = new B1Module();
			B1Module.DataType = B1Client.DataTypeT.Plain;
			B1Module.HeaderType = B1Client.HeaderTypeT.A;
			B1Module.Response += PackedReceivedEventHandler;
			B1Module.Request += PacketSendRequestEventHandler;
			
			ComPort = new SerialPort();
			ComPort.Parity = Parity.None;
			ComPort.StopBits = StopBits.One;
			ComPort.DataBits = 8;
			ComPort.BaudRate = 9600;
			ComPort.Handshake = Handshake.None;
			ComPort.RtsEnable = false;
			ComPort.ReceivedBytesThreshold = 1;
			ComPort.DataReceived += ComPortDataReceivedEventHandler;
			//
			// TODO: Add constructor code after the InitializeComponent() call.
			//
		}
		
		void COMPortsAvailabe()
		{
			String[] ports = SerialPort.GetPortNames();
			cbCOMPort.Items.AddRange(ports);					
		}
		
		private void CloseSerialOnExit(){
			try{
				ComPort.Close();
			}
			catch (Exception ex){
				MessageBox.Show(ex.Message);
			}
		}
		
		private void ComPortDataReceivedEventHandler(object sender, SerialDataReceivedEventArgs e) {
	        byte[] RxData = new byte[ComPort.BytesToRead];
	        ComPort.Read(RxData, 0, RxData.Length);	        
	        B1Module.SupplyRxData(RxData);
	    }
		
		private string FormatResponse(ResponseArgs Packet) {
			return "[RX]: " + Packet.Response.ToString() + GetParameters(Packet) + Environment.NewLine;
	    }
		
		private string GetParameters(ResponseArgs Packet) {
	            return " {" + ByteArrayToString(Packet.Parameters) + "}";
	    }
		
		public static string ByteArrayToString(byte[] ba) {
	        if (ba == null || ba.Length == 0) {
	            return "";
	        }
	        
            StringBuilder hex = new StringBuilder(ba.Length * 2);
            foreach (byte b in ba) {
                hex.AppendFormat("{0:X2}", b);
            }
            return hex.ToString();
        }
		
		public static string ByteArrayToString(byte[] ba, byte StartCell, byte NumberOfCells) {
	        if (ba == null || ba.Length == 0) {
	            return "";
	        }
	        
            StringBuilder hex = new StringBuilder(NumberOfCells);
            for (int i = StartCell; i < (StartCell + NumberOfCells); i++){
            	hex.AppendFormat("{0:X2}", ba[i]);
            }
            return hex.ToString();
        }
		
		private void PackedReceivedEventHandler(object sender, EventArgs Packet) {
	        if (InvokeRequired) {
	            try {
	               Invoke((MethodInvoker)delegate {
	                   PackedReceived((ResponseArgs)Packet);
                    });
	            } catch {
	                return;
	            }
	        } else {
	           PackedReceived((ResponseArgs)Packet);
	        }
	    }

		
		private void PackedReceived(ResponseArgs Packet) {
	       
	       
	       if (Packet.Response == B1Client.ResponseT.ACK){
	       	
	       switch(lastCommand){

	       		case ReadUidFromMemory:
	       			tbTagUID.Text = FormatResponse(Packet).Substring(11, Convert.ToInt32(Packet.Parameters[11])*2);		//cut UID and display in on the screen

	       			printTagType(Packet.Parameters[10]);
	       			TableUID[0] = Packet.Parameters[11]; //copy UID size to the beginning
	       			
	       			if (Packet.Parameters[10] > 0 && Packet.Parameters[10] <= 16){ //if any tag has been found
	       				SearchStarted = false;
	       				btnSearch.Text = "Search";
	       				btnConnect.Enabled = true;
	       				tbTagType.Enabled = true;
	       				
	       			}
	       			
	       			// Unknown tag
	       			if (Packet.Parameters[10] == 16){
	       				for (int i=0; i < Packet.Parameters[11]; i++){
	       					TableUID[i+1] = Packet.Parameters[i];
	       				}
	       			}
	       			// NTAG216 or NTAG216F?
					if (Packet.Parameters[10] == 15 || Packet.Parameters[10] == 12) {
						for (int i=0; i< Packet.Parameters[11] ; i++){
							TableUID[i+1] = Packet.Parameters[i];
						}					
					}	
					// NTAG 215
					else if(Packet.Parameters[10] == 14){
						for (int i=0; i< Packet.Parameters[11] ; i++){
							TableUID[i+1] = Packet.Parameters[i];
						}
					}
					// NTAG 213, NTAG213F, NTAG203F
					else if(Packet.Parameters[10] == 13 ||
							Packet.Parameters[10] == 11 ||
							Packet.Parameters[10] == 8 ){
						for (int i=0; i< Packet.Parameters[11] ; i++){
							TableUID[i+1] = Packet.Parameters[i];
						}
					}
					// NTAG 212
					else if(Packet.Parameters[10] == 10 ||
					        Packet.Parameters[10] == 4){
						for (int i=0; i< Packet.Parameters[11] ; i++){
							TableUID[i+1] = Packet.Parameters[i];
						}
					}
					// NTAG210
					else if(Packet.Parameters[10] == 9 ||
					       	Packet.Parameters[10] == 2 ||
					       	Packet.Parameters[10] == 3){
						for (int i=0; i< Packet.Parameters[11] ; i++){
							TableUID[i+1] = Packet.Parameters[i];
						}
					}
					// Mifare 1k
					else if(Packet.Parameters[10] == 6){
						for (int i=0; i< Packet.Parameters[11] ; i++){
							TableUID[i+1] = Packet.Parameters[i];
						}
					}
					// Mifare Mini
					else if(Packet.Parameters[10] == 5){
						for (int i=0; i< Packet.Parameters[11] ; i++){
							TableUID[i+1] = Packet.Parameters[i];
						}
					}
					//Mifare Classic 4k
					else if(Packet.Parameters[10] == 7){
						for (int i=0; i< Packet.Parameters[11] ; i++){
							TableUID[i+1] = Packet.Parameters[i];
						}					
					}
					// if there is no tag in the field 
					else if(Packet.Parameters[10] == 0) {
						byte[] command = new byte[1];
						command[0] = 0x01;
						B1Module.SendWriteToRFIDMemoryCommand((ushort)UartCommandT.WriteToRfidMemory, command);  // Get UID and Type
						lastCommand = GetUID;
						break;
					}
					
					Array.Resize(ref TableUID, (int)(Packet.Parameters[11]+1)); // resize TableUID size <UIDSize><UID>
					
	       		break;

	       	}
	       }
	       if (Packet.Response == B1Client.ResponseT.AsyncPacket && 
	           Packet.Parameters.Length == 1 && 
	           (Packet.Parameters[0] & 0x20) != 0) {
	       	
	       switch(lastCommand){
	       		
	       		case GetUID:
	       			//Read UID from memory
	       			B1Module.SendReadFromRFIDMemoryCommand(0x0014, 12);
	       			lastCommand = ReadUidFromMemory;
	       		break;
	       		     		    		
	       	}
	       }
	       	
		}
		
		private void PacketSendRequestEventHandler(object sender, EventArgs Request) {
	        try {
	           ComPort.Write(((B1Client.RequestArgs)Request).Data, 0, ((B1Client.RequestArgs)Request).Data.Length);
	        } catch {
	            MessageBox.Show("Unable to send via Com Port");
	        }
	    }
			
				
		void btnConnectClick(object sender, EventArgs e)	{
		
			if (ComPort.IsOpen) {
				btnConnect.Text = "Connect";
				cbCOMPort.Enabled = true;
				btnSearch.Enabled = false;
				tbTagUID.Enabled = false;
				tbTagType.Enabled = false;
				progressBar1.Value = 0;
						
				try{
					ComPort.Close();
				} catch {
					MessageBox.Show("Error during disconnecting");
				}
			} else {
				if (btnConnect.Text == "Disconnect") {
					btnConnect.Text = "Connect";
					cbCOMPort.Enabled = true;

					
				} else if (cbCOMPort.SelectedIndex == -1){
					btnConnect.Text = "Connect";
					cbCOMPort.Enabled = true;
					MessageBox.Show("Please select the proper COM Port and enter the password");
				}
				else{
					try {
						ComPort.PortName = (string)cbCOMPort.SelectedItem;
						ComPort.Open();
						progressBar1.Value = 100;
						btnConnect.Text = "Disconnect";
						btnSearch.Enabled = true;
						tbTagUID.Enabled = true;
						cbCOMPort.Enabled = false;
						
						//Send GET UID command (0x01)
						byte[] command = new byte[1];
						command[0] = 0x01;
						B1Module.SendWriteToRFIDMemoryCommand((ushort)UartCommandT.WriteToRfidMemory, command);
						lastCommand = ReadUidFromMemory;						
					} 
					catch {
						btnConnect.Text = "Connect";
						cbCOMPort.Enabled = true;
						cbCOMPort.SelectedIndex = -1;
						MessageBox.Show("Unable to open port.");
					}
				}
			}
		}
		

		private class ExecuteEventArgs : EventArgs {
	        public object[] Parameters;
	        
	        public ExecuteEventArgs(int NoParameters) : base () {
	            Parameters = new object[NoParameters];
	        }
	        
	        public ExecuteEventArgs() : base () {
	            Parameters = null;
            }
	    }
		
		private void printTagType(byte tagType)
		{
			switch (tagType)
			{
	                case 1: tbTagType.Text = "incomplete type (0x01)"; 		break;
                    case 2: tbTagType.Text = "Ultralight (0x02)";      		break;
                    case 3: tbTagType.Text = "Ultralight EV1 80B (0x03)"; 	break; 
                    case 4: tbTagType.Text = "Ultralight EV1 164B (0x04)";  break;
                    case 5: tbTagType.Text = "Classic Mini (0x05)";         break;
                    case 6: tbTagType.Text = "Classic 1K (0x06)";  			break;  
                    case 7: tbTagType.Text = "Classic 4K (0x07)";			break;    
                    case 8: tbTagType.Text = "NTAG203F (0x08)";             break;     
                    case 9: tbTagType.Text = "NTAG210 (0x09)";              break;
                    case 10:tbTagType.Text = "NTAG212 (0x0A)";				break;
                    case 11:tbTagType.Text = "NTAG213F (0x0B)";				break;
                    case 12:tbTagType.Text = "NTAG216F (0x0C)";				break;
                    case 13:tbTagType.Text = "NTAG213 (0x0D)";				break;
                    case 14:tbTagType.Text = "NTAG215 (0x0E)";              break;     
                    case 15:tbTagType.Text = "NTAG216 (0x0F)";				break;
                    case 16:tbTagType.Text = "Unknown (0x10)";				break;
                    default:tbTagType.Text = "";
                            tbTagUID.Text  = "Searching for a TAG...";		break;
			}
		}
		
		
		void BtnSearchClick(object sender, EventArgs e)
		{
			if (SearchStarted){
				SearchStarted = false;
				btnConnect.Enabled = true;
				btnSearch.Text = "Search";
				tbTagUID.Text = "";
				lastCommand = 0;
			}
			else {
				SearchStarted = true;
				btnSearch.Text = "Stop searching";
				btnConnect.Enabled = false;
				
				// Send GET UID Command
				byte[] command = new byte[1];
				command[0] = 0x01;
				B1Module.SendWriteToRFIDMemoryCommand((ushort)UartCommandT.WriteToRfidMemory, command);
				lastCommand = GetUID;
			}		
		}

		void CbCOMPortDropDown(object sender, EventArgs e)
		{
			cbCOMPort.Items.Clear();
			String[] ports = SerialPort.GetPortNames(); 
			cbCOMPort.Items.AddRange(ports);				
		}
		
	} // end of class MainForm 

}